
"""
Machine Learning library for academic purposes.
GitHub: https://github.com/guiferviz/happyml-py
"""


__version__ = '0.0.3'


def greet():
    """
    HappyML is so friendly that greets you!
    Prints a happy message to the standard output.
    """
    print("Those about to learn we salute you :)")

